#ifndef TKXSBASE_PCH_H
#define TKXSBASE_PCH_H

#include <Interface_Check.hxx>
#include <Interface_CheckIterator.hxx>
#include <Interface_EntityIterator.hxx>
#include <Interface_FileReaderData.hxx>
#include <Interface_Graph.hxx>

#endif // TKXSBASE_PCH_H
